@@ -6,13 +6,13 @@ use strict;
 use 5.008003;
 no warnings 'utf8';
 
-our $VERSION = '2.006';
+our $VERSION = '2.007';
 
 use List::MoreUtils qw( any first_index );
 
 use Term::Choose     qw( choose );
 use Term::Form       qw();
-#use Term::TablePrint qw(print_table);
+use Term::TablePrint qw(print_table);
 
 use if $^O eq 'MSWin32', 'Win32::Console::ANSI';
 
@@ -24,28 +24,26 @@ use App::DBBrowser::Table::Substatements;
 
 
 sub new {
-    my ( $class, $info, $opt ) = @_;
-    bless { i => $info, o => $opt }, $class;
+    my ( $class, $info, $options, $data ) = @_;
+    bless {
+        i => $info,
+        o => $options,
+        d => $data
+    }, $class;
 }
 
 
 sub on_table {
-    my ( $sf, $sql, $dbh ) = @_;
+    my ( $sf, $sql ) = @_;
     my $stmt_h = Term::Choose->new( $sf->{i}{lyt_stmt_h} );
-    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o} );
-    my $sb = App::DBBrowser::Table::Substatements->new( $sf->{i}, $sf->{o} );
-    my $sub_stmts = {
-        Select => [ qw( print_tbl columns aggregate distinct where group_by having order_by limit functions lock ) ],
-        Delete => [ qw( commit     where functions ) ],
-        Update => [ qw( commit set where functions ) ],
-    };
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my $sb = App::DBBrowser::Table::Substatements->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my $sub_stmts = [ qw( print_tbl columns aggregate distinct where group_by having order_by limit functions lock ) ];
     my $lk = [ '  Lk0', '  Lk1' ];
     my %cu = (
-        commit          => '  CONFIRM Stmt',
         hidden          => 'Customize:',
         print_tbl       => 'Print TABLE',
         columns         => '- SELECT',
-        set             => '- SET',
         aggregate       => '- AGGREGATE',
         distinct        => '- DISTINCT',
         where           => '- WHERE',
@@ -56,43 +54,23 @@ sub on_table {
         lock            => $lk->[$sf->{i}{lock}],
         functions       => '  Func',
     );
-    my @aggregate = ( "AVG(X)", "COUNT(X)", "COUNT(*)", "MAX(X)", "MIN(X)", "SUM(X)" ); #
-    my ( $DISTINCT, $ALL, $ASC, $DESC, $AND, $OR ) = ( "DISTINCT", "ALL", "ASC", "DESC", "AND", "OR" ); #
     if ( $sf->{i}{lock} == 0 ) {
         $ax->reset_sql( $sql );
     }
     my $stmt_type = 'Select';
-    my $backup_sql;
     my $old_idx = 1;
-    my @pre = ( undef, $sf->{i}{ok} );
 
     CUSTOMIZE: while ( 1 ) {
-        $backup_sql = $ax->backup_href( $sql ) if $stmt_type eq 'Select'; #
-        my $choices = [ $cu{hidden}, undef, @cu{@{$sub_stmts->{$stmt_type}}} ];
+        my $choices = [ $cu{hidden}, undef, @cu{@$sub_stmts} ];
         $ax->print_sql( $sql, [ $stmt_type ] );
         # Choose
         $ENV{TC_RESET_AUTO_UP} = 0;
         my $idx = choose(
             $choices,
-            { %{$sf->{i}{lyt_stmt_v}}, prompt => '', index => 1, default => $old_idx,
-            undef => $stmt_type ne 'Select' ? $sf->{i}{_back} : $sf->{i}{back} }
+            { %{$sf->{i}{lyt_stmt_v}}, prompt => '', index => 1, default => $old_idx, undef => $sf->{i}{back} }
         );
         if ( ! defined $idx || ! defined $choices->[$idx] ) {
-            if ( $stmt_type eq 'Select'  ) {
-                last CUSTOMIZE;
-            }
-            elsif( $stmt_type eq 'Delete' || $stmt_type eq 'Update' ) {
-                if ( $sql->{where_stmt} || $sql->{set_stmt} ) {
-                    $ax->reset_sql( $sql );
-                }
-                else {
-                    $stmt_type = 'Select';
-                    $old_idx = 1;
-                    $sql = $backup_sql;
-                }
-                next CUSTOMIZE;
-            }
-            else { die $stmt_type }
+            last CUSTOMIZE;
         }
         my $custom = $choices->[$idx];
         if ( $sf->{o}{G}{menu_memory} ) {
@@ -116,13 +94,13 @@ sub on_table {
             if ( ! ( $sql->{select_type} eq '*' || $sql->{select_type} eq 'chosen_cols' ) ) {
                 $ax->reset_sql( $sql );
             }
-            my $tmp = $sb->columns( $dbh, $stmt_h, $sql, $stmt_type );
+            my $tmp = $sb->columns( $stmt_h, $sql, $stmt_type );
             if ( defined $tmp ) {
                 $sql->{$_} = $tmp->{$_} for keys %$tmp;
             }
         }
         elsif ( $custom eq $cu{'distinct'} ) {
-            my $tmp = $sb->distinc( $stmt_h, $sql, $stmt_type );
+            my $tmp = $sb->distinct( $stmt_h, $sql, $stmt_type );
             if ( defined $tmp ) {
                 $sql->{$_} = $tmp->{$_} for keys %$tmp;
             }
@@ -131,13 +109,7 @@ sub on_table {
             if ( $sql->{select_type} eq '*' || $sql->{select_type} eq 'chosen_cols' ) {
                 $ax->reset_sql( $sql );
             }
-            my $tmp = $sb->aggregate( $dbh, $stmt_h, $sql, $stmt_type );
-            if ( defined $tmp ) {
-                $sql->{$_} = $tmp->{$_} for keys %$tmp;
-            }
-        }
-        elsif ( $custom eq $cu{'set'} ) {
-            my $tmp = $sb->set( $stmt_h, $sql, $stmt_type );
+            my $tmp = $sb->aggregate( $stmt_h, $sql, $stmt_type );
             if ( defined $tmp ) {
                 $sql->{$_} = $tmp->{$_} for keys %$tmp;
             }
@@ -175,44 +147,36 @@ sub on_table {
                 $sql->{$_} = $tmp->{$_} for keys %$tmp;
             }
         }
-        elsif ( $custom eq $cu{'hidden'} ) { # insert/update/delete
-            $stmt_type = $sf->__table_write_access( $sql, $stmt_type );
-            if ( $stmt_type eq 'Insert' ) {
-                require App::DBBrowser::Table::Insert;
-                my $tbl_in = App::DBBrowser::Table::Insert->new( $sf->{i}, $sf->{o} );
-                my $ok = $tbl_in->build_insert_stmt( $sql, [ $stmt_type ], $dbh );
-                if ( $ok ) {
-                    $ok = $sf->commit_sql( $sql, [ $stmt_type ], $dbh );
-                }
-                $stmt_type = 'Select';
-                $sql = $backup_sql;
-                next CUSTOMIZE;
-            }
+        elsif ( $custom eq $cu{'hidden'} ) {
+            my $backup_sql = $ax->backup_href( $sql );
+            $sf->__table_write_access( $sql );
+            $stmt_type = 'Select';
             $old_idx = 1;
+            $sql = $backup_sql;
         }
         elsif ( $custom eq $cu{'functions'} ) {
-            my $nh = App::DBBrowser::Table::Functions->new( $sf->{i}, $sf->{o} );
-            my $ok = $nh->col_function( $dbh, $sql, $stmt_type ); #
+            my $nh = App::DBBrowser::Table::Functions->new( $sf->{i}, $sf->{o}, $sf->{d} );
+            my $backup_sql = $ax->backup_href( $sql );
+            my $ok = $nh->col_function( $sql, $stmt_type );
             if ( ! $ok ) {
                 $sql = $backup_sql;
-                next CUSTOMIZE;
             }
         }
         elsif ( $custom eq $cu{'print_tbl'} ) {
             my $cols_sql = " ";
             if ( $sql->{select_type} eq '*' ) {
-                if ( $sf->{i}{multi_tbl} eq 'join' ) {          # ?
-                    $cols_sql .= join( ', ', @{$sql->{cols}} ); #
-                }                                               #
-                else {
+                #if ( $sf->{i}{multi_tbl} eq 'join' ) {          # ?
+                #    $cols_sql .= join( ', ', @{$sql->{cols}} ); #
+                #}                                               #
+                #else {
                     $cols_sql .= "*";
-                }
+                #}
             }
             elsif ( $sql->{select_type} eq 'chosen_cols' ) {
-                $cols_sql .= $ax->__cols_as_string( $sql, 'chosen_cols' );
+                $cols_sql .= $ax->cols_as_string( $sql, 'chosen_cols' );
             }
             elsif ( @{$sql->{aggr_cols}} || @{$sql->{group_by_cols}} ) {
-                $cols_sql .= $ax->__cols_as_string( $sql, 'aggr_and_group_by_cols' ); #
+                $cols_sql .= $ax->cols_as_string( $sql, 'aggr_and_group_by_cols' ); #
             }
             #else {
             #    $cols_sql .= "*";
@@ -232,8 +196,8 @@ sub on_table {
             else {
                 $sf->{o}{table}{max_rows} = 0;
             }
-            my @arguments = ( @{$sql->{select_sq_args}}, @{$sql->{where_args}}, @{$sql->{having_args}} ); # select_args
-            if ( $sf->{o}{G}{expert_subqueries} ) {
+            my @arguments = ( @{$sql->{select_sq_args}}, @{$sql->{where_args}}, @{$sql->{having_args}} );
+            if ( $sf->{o}{G}{subqueries} ) {
                 my $tmp;
                 if ( @{$sf->{i}{stmt_history}||[]} ) {
                     if ( $select . join( ',', @arguments ) ne $sf->{i}{stmt_history}[0] . join( ',', @{$sf->{i}{stmt_history}[1]||[]} ) ) {
@@ -251,42 +215,139 @@ sub on_table {
             }
             local $| = 1;
             print $sf->{i}{clear_screen};
-            my $sth = $dbh->prepare( $select );
+            my $sth = $sf->{d}{dbh}->prepare( $select );
             print 'Execute ...' . "\n" if $sf->{o}{table}{progress_bar};
             $sth->execute( @arguments );
             my $col_names = $sth->{NAME}; # not quoted
             my $all_arrayref = $sth->fetchall_arrayref;
             unshift @$all_arrayref, $col_names;
             print $sf->{i}{clear_screen};
-            # return $sql explicitly since after a restore backup refers to a different hash. ?
+            # return $sql explicitly since after a restore backup it refers to a different hash. ?
             return $all_arrayref, $sql;
         }
-        elsif ( $custom eq $cu{'commit'} ) {
-            my $ok = $sf->commit_sql( $sql, [ $stmt_type ], $dbh );
-            $stmt_type = 'Select';
-            $old_idx = 1;
-            $sql = $backup_sql;
-            next CUSTOMIZE;
-        }
         else {
             die "$custom: no such value in the hash \%cu";
         }
     }
-    return;
+}
+
+
+sub __table_write_access {
+    my ( $sf, $sql ) = @_;
+    my $stmt_h = Term::Choose->new( $sf->{i}{lyt_stmt_h} );
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my $sb = App::DBBrowser::Table::Substatements->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my @stmt_types;
+    if ( ! $sf->{i}{multi_tbl} ) {
+        push @stmt_types, 'Insert' if $sf->{o}{G}{insert_ok};
+        push @stmt_types, 'Update' if $sf->{o}{G}{update_ok};
+        push @stmt_types, 'Delete' if $sf->{o}{G}{delete_ok};
+    }
+    elsif ( $sf->{i}{multi_tbl} eq 'join' && $sf->{d}{driver} eq 'mysql' ) {
+        push @stmt_types, 'Update' if $sf->{o}{G}{update_ok};
+    }
+    if ( ! @stmt_types ) {
+        return;
+    }
+    STMT_TYPE: while ( 1 ) {
+        # Choose
+        my $stmt_type = choose(
+            [ undef, map( "- $_", @stmt_types ) ],
+            { %{$sf->{i}{lyt_3}}, prompt => 'Choose SQL type:' }
+        );
+        if ( ! defined $stmt_type ) {
+            return;
+        }
+        $stmt_type =~ s/^-\ //;
+        $ax->reset_sql( $sql );
+        if ( $stmt_type eq 'Insert' ) {
+            require App::DBBrowser::Table::Insert;
+            my $tbl_in = App::DBBrowser::Table::Insert->new( $sf->{i}, $sf->{o}, $sf->{d} );
+            my $ok = $tbl_in->build_insert_stmt( $sql, [ $stmt_type ] );
+            if ( $ok ) {
+                $ok = $sf->commit_sql( $sql, [ $stmt_type ] );
+            }
+            next STMT_TYPE;
+        }
+        my $sub_stmts = {
+            Delete => [ qw( commit     where functions ) ],
+            Update => [ qw( commit set where functions ) ],
+        };
+        my %cu = (
+            commit    => '  CONFIRM Stmt',
+            set       => '- SET',
+            where     => '- WHERE',
+            functions => '  Func',
+        );
+        my $old_idx = 0;
+
+        CUSTOMIZE: while ( 1 ) {
+            my $choices = [ undef, @cu{@{$sub_stmts->{$stmt_type}}} ];
+            $ax->print_sql( $sql, [ $stmt_type ] );
+            # Choose
+            $ENV{TC_RESET_AUTO_UP} = 0;
+            my $idx = choose(
+                $choices,
+                { %{$sf->{i}{lyt_stmt_v}}, prompt => 'Customize:', index => 1, default => $old_idx, undef => $sf->{i}{_back} }
+            );
+            if ( ! defined $idx || ! defined $choices->[$idx] ) {
+                next STMT_TYPE;
+            }
+            my $custom = $choices->[$idx];
+            if ( $sf->{o}{G}{menu_memory} ) {
+                if ( $old_idx == $idx && ! $ENV{TC_RESET_AUTO_UP} ) {
+                    $old_idx = 0;
+                    next CUSTOMIZE;
+                }
+                else {
+                    $old_idx = $idx;
+                }
+            }
+            delete $ENV{TC_RESET_AUTO_UP};
+            if ( $custom eq $cu{'set'} ) {
+                my $tmp = $sb->set( $stmt_h, $sql, $stmt_type );
+                if ( defined $tmp ) {
+                    $sql->{$_} = $tmp->{$_} for keys %$tmp;
+                }
+            }
+            elsif ( $custom eq $cu{'where'} ) {
+                my $tmp = $sb->where( $stmt_h, $sql, $stmt_type );
+                if ( defined $tmp ) {
+                    $sql->{$_} = $tmp->{$_} for keys %$tmp;
+                }
+            }
+            elsif ( $custom eq $cu{'functions'} ) {
+                my $nh = App::DBBrowser::Table::Functions->new( $sf->{i}, $sf->{o}, $sf->{d} );
+                my $backup_sql = $ax->backup_href( $sql ); ##
+                my $ok = $nh->col_function( $sql, $stmt_type );
+                if ( ! $ok ) {
+                    $sql = $backup_sql;
+                }
+            }
+            elsif ( $custom eq $cu{'commit'} ) {
+                my $ok = $sf->commit_sql( $sql, [ $stmt_type ] );
+                next STMT_TYPE;
+            }
+            else {
+                die "$custom: no such value in the hash \%cu";
+            }
+        }
+    }
 }
 
 
 sub commit_sql {
-    my ( $sf, $sql, $stmt_typeS, $dbh ) = @_;
-    my $ax  = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o} );
+    my ( $sf, $sql, $stmt_typeS ) = @_;
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $stmt_v = Term::Choose->new( $sf->{i}{lyt_stmt_v} );
+    my $dbh = $sf->{d}{dbh};
     local $| = 1;
     print $sf->{i}{clear_screen};
     print 'DB work ...' . "\n" if $sf->{o}{table}{progress_bar}; #
     my $transaction;
     eval { $transaction = $dbh->begin_work } or do { $dbh->{AutoCommit} = 1; $transaction = 0 };
     my $rows_to_execute = [];
-    #my $row_count;
+    my $row_count;
     my $stmt;
     my $stmt_type = $stmt_typeS->[-1];
     if ( $stmt_type eq 'Insert' ) {
@@ -298,7 +359,7 @@ sub commit_sql {
                             join( ', ', @{$sql->{insert_into_cols}} ),
                             join( ', ', ( '?' ) x @{$sql->{insert_into_cols}} );
         $rows_to_execute = $sql->{insert_into_args};
-        #$row_count = @$rows_to_execute;
+        $row_count = @$rows_to_execute;
     }
     else {
         my %map_stmt_types = (
@@ -311,20 +372,34 @@ sub commit_sql {
         $stmt .= $sql->{set_stmt}      if $sql->{set_stmt};
         $stmt .= $sql->{where_stmt}    if $sql->{where_stmt};
         $rows_to_execute->[0] = [ @{$sql->{set_args}}, @{$sql->{where_args}} ];
-        #if ( ! eval {
-        #    my $sth = $dbh->prepare( "SELECT * FROM " . $sql->{table} . ( $sql->{where_stmt} ? $sql->{where_stmt} : '' ) );
-        #    $sth->execute( @{$sql->{where_args}} );
-        #    my $col_names = $sth->{NAME};
-        #    my $all_arrayref = $sth->fetchall_arrayref;
-        #    $row_count = @$all_arrayref;
-        #    unshift @$all_arrayref, $col_names;
-        #    my $prompt_pt = "ENTER to continue\n$stmt_type - affected records:\n";
-        #    print_table( $all_arrayref, { %{$sf->{o}{table}}, prompt => $prompt_pt, max_rows => 0, table_expand => 0 } ); #
-        #    die "'row_count' undefined" if ! defined $row_count;
-        #    1 }
-        #) {
-        #    $ax->print_error_message( "$@Fetching info: affected records ...\n", $stmt_type );
-        #}
+        if ( ! eval {
+            my $sth = $dbh->prepare( "SELECT * FROM " . $sql->{table} . ( $sql->{where_stmt} ? $sql->{where_stmt} : '' ) );
+            $sth->execute( @{$sql->{where_args}} );
+            my $col_names = $sth->{NAME};
+            my $all_arrayref = $sth->fetchall_arrayref;
+            $row_count = @$all_arrayref;
+            unshift @$all_arrayref, $col_names;
+            my $prompt_pt;
+            if ( $stmt_type eq 'Update' ) {
+                my $filled = $sql->{set_stmt};
+                for my $val ( @{$sql->{set_args}} ) {
+                    $filled =~ s/\?/$val/;
+                }
+                $filled =~ s/^\s+//;
+                $prompt_pt = "i: These records will be updated with [$filled]\n";
+            }
+            else {
+                $prompt_pt = "i: These records will be deleted\n";
+            }
+            if ( @$all_arrayref > 1 ) {
+                print_table( $all_arrayref, { %{$sf->{o}{table}}, prompt => $prompt_pt, max_rows => 0, keep_header => 0 } ); #
+            }
+            die "'row_count' undefined" if ! defined $row_count;
+            1 }
+        ) {
+            $ax->print_error_message( "$@Fetching info: affected records ...\n", $stmt_type );
+        }
+
     }
     if ( $transaction ) {
         my $rolled_back;
@@ -333,7 +408,9 @@ sub commit_sql {
             for my $values ( @$rows_to_execute ) {
                 $sth->execute( @$values );
             }
-            my $row_count = $stmt_type eq 'Insert' ? @$rows_to_execute : $sth->rows;
+            if ( ! defined $row_count ) {
+                $row_count = $sth->rows;
+            }
             my $commit_ok = sprintf qq(  %s %d "%s"), 'COMMIT', $row_count, $stmt_type;
             $ax->print_sql( $sql, $stmt_typeS );
             # Choose
@@ -359,18 +436,10 @@ sub commit_sql {
         return 1;
     }
     else {
-#
-        my $row_count;
-        if ( $stmt_type eq 'Insert' ) {
-            $row_count = @$rows_to_execute;
-        }
-        else {
-            my $count_stmt;
-            $count_stmt .= "SELECT COUNT(*) FROM " . $sql->{table};
-            $count_stmt .= $sql->{where_stmt};
+        if ( ! defined $row_count ) {
+            my $count_stmt = "SELECT COUNT(*) FROM " . $sql->{table} . $sql->{where_stmt};
             ( $row_count ) = $dbh->selectrow_array( $count_stmt, undef, @{$sql->{where_args}} );
         }
-#
         my $commit_ok = sprintf qq(  %s %d "%s"), 'EXECUTE', $row_count, $stmt_type;
         $ax->print_sql( $sql, $stmt_typeS ); #
         # Choose
@@ -396,34 +465,6 @@ sub commit_sql {
 }
 
 
-sub __table_write_access {
-    my ( $sf, $sql, $stmt_type ) = @_;
-    my @stmt_types;
-    if ( ! $sf->{i}{multi_tbl} ) {
-        push @stmt_types, 'Insert' if $sf->{o}{G}{insert_ok};
-        push @stmt_types, 'Update' if $sf->{o}{G}{update_ok};
-        push @stmt_types, 'Delete' if $sf->{o}{G}{delete_ok};
-    }
-    elsif ( $sf->{i}{multi_tbl} eq 'join' && $sf->{i}{driver} eq 'mysql' ) {
-        push @stmt_types, 'Update' if $sf->{o}{G}{update_ok};
-    }
-    if ( ! @stmt_types ) {
-        return $stmt_type; #
-    }
-    # Choose
-    my $type_choice = choose(
-        [ undef, map( "- $_", @stmt_types ) ],
-        { %{$sf->{i}{lyt_3}}, prompt => 'Choose SQL type:' }
-    );
-    if ( defined $type_choice ) {
-        ( $stmt_type = $type_choice ) =~ s/^-\ //;
-        my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o} );
-        $ax->reset_sql( $sql );
-    }
-    return $stmt_type;
-}
-
-
 1;
 
 
