@@ -6,7 +6,7 @@ use strict;
 use 5.008003;
 no warnings 'utf8';
 
-our $VERSION = '2.006';
+our $VERSION = '2.007';
 
 use Cwd                   qw( realpath );
 use Encode                qw( encode decode );
@@ -23,24 +23,30 @@ use Term::Choose       qw( choose );
 use Term::Choose::Util qw( choose_a_file choose_a_subset );
 use Term::Form         qw();
 
+use if $^O eq 'MSWin32', 'Win32::Console::ANSI';
+
 use App::DBBrowser::Auxil;
 use App::DBBrowser::Opt;
-use App::DBBrowser::Table;
 
 use open ':encoding(locale)';
 
 
 sub new {
-    my ( $class, $info, $opt ) = @_;
-    my $sf = { i => $info, o => $opt };
-    $sf->{i}{tmp_copy_paste} = catfile $info->{app_dir}, 'c0py_and_pasTe_tmp_file.csv';
+    my ( $class, $info, $options, $data ) = @_;
+    my $sf = {
+        i => $info,
+        o => $options,
+        d => $data,
+        tmp_copy_paste => catfile( $info->{app_dir}, 'Copy_and_Paste_tmp_file.csv' ),
+        input_files    => catfile( $info->{app_dir}, 'file_history.txt' )
+    };
     bless $sf, $class;
 }
 
 
 sub __insert_into_stmt_cols {
     my ( $sf, $sql, $stmt_typeS ) = @_;
-    my $ax  = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o} );
+    my $ax  = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     $sql->{insert_into_cols} = [];
     my @cols = ( @{$sql->{cols}} );
 
@@ -86,9 +92,9 @@ sub __insert_into_stmt_cols {
 
 
 sub build_insert_stmt {
-    my ( $sf, $sql, $stmt_typeS, $dbh ) = @_;
-    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o} );
-    my $obj_db = App::DBBrowser::DB->new( $sf->{i}, $sf->{o} );
+    my ( $sf, $sql, $stmt_typeS ) = @_;
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my $plui = App::DBBrowser::DB->new( $sf->{i}, $sf->{o} );
     $ax->reset_sql( $sql );
     my @cu_keys = ( qw/insert_col insert_copy insert_file settings/ );
     my %cu = (
@@ -122,8 +128,8 @@ sub build_insert_stmt {
         }
         delete $ENV{TC_RESET_AUTO_UP};
         if ( $custom eq $cu{settings} ) {
-            my $obj_opt = App::DBBrowser::Opt->new( $sf->{i}, $sf->{o} );
-            $obj_opt->config_insert;
+            my $opt = App::DBBrowser::Opt->new( $sf->{i}, $sf->{o} );
+            $opt->config_insert;
             next MENU;
         }
         my $cols_ok = $sf->__insert_into_stmt_cols( $sql, $stmt_typeS );
@@ -150,7 +156,7 @@ sub build_insert_stmt {
 
 sub __from_col_by_col {
     my ( $sf, $sql, $stmt_typeS ) = @_;
-    my $ax  = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o} );
+    my $ax  = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     $sql->{insert_into_args} = [];
     my $trs = Term::Form->new();
 
@@ -210,8 +216,8 @@ sub __file_name {
 
     FILE: while ( 1 ) {
         my @files;
-        if ( $sf->{o}{insert}{max_files} && -e $sf->{i}{input_files} ) {
-            open my $fh_in, '<', $sf->{i}{input_files} or die $!;
+        if ( $sf->{o}{insert}{max_files} && -e $sf->{input_files} ) {
+            open my $fh_in, '<', $sf->{input_files} or die $!;
             while ( my $fl = <$fh_in> ) {
                 chomp $fl;
                 next if ! -e $fl;
@@ -232,7 +238,7 @@ sub __file_name {
         }
         if ( $file eq $add_file ) {
             my $prompt = sprintf "%s", $sf->__parse_setting( 'file' );
-            my $dir = $sf->{i}{tmp_files_dir} || $sf->{i}{home_dir};
+            my $dir = $sf->{tmp_files_dir} || $sf->{i}{home_dir};
             # Choose_a_file
             $file = choose_a_file( { dir => $dir, mouse => $sf->{o}{table}{mouse} } );
             if ( ! defined $file || ! length $file ) {
@@ -245,13 +251,13 @@ sub __file_name {
                 while ( @files > $sf->{o}{insert}{max_files} ) {
                     shift @files;
                 }
-                open my $fh_out, '>', $sf->{i}{input_files} or die $!;
+                open my $fh_out, '>', $sf->{input_files} or die $!;
                 for my $fl ( @files ) {
                     print $fh_out $fl . "\n";
                 }
                 close $fh_out;
             }
-            $sf->{i}{tmp_files_dir} = dirname $file;
+            $sf->{tmp_files_dir} = dirname $file;
             return $file;
         }
         elsif ( $file eq $del_file ) {
@@ -265,7 +271,7 @@ sub __file_name {
             if ( ! defined $idx || ! @$idx ) {
                 next FILE;
             }
-            open my $fh_out, '>', $sf->{i}{input_files} or die $!; # file_name
+            open my $fh_out, '>', $sf->{input_files} or die $!; # file_name
             for my $i ( 0 .. $#files ) {
                 if ( any { $i == $_ } @$idx ) {
                     next;
@@ -292,7 +298,7 @@ sub __parse_setting {
     elsif ( $i == 1 ) {
         $sep = $sf->{o}{split}{i_f_s};
     }
-    my $str = "($parse_mode";
+    my $str = "(Tool: $parse_mode";
     $str .= ", sep: $sep" if defined $sep;
     $str .= ")";
     return $str;
@@ -304,8 +310,8 @@ sub from_copy_and_paste {
     print $sf->{i}{clear_screen};
     my $prompt = sprintf "Mulit row  %s:\n", $sf->__parse_setting( 'copy_and_paste' );
     print $prompt;
-    my $file = $sf->{i}{tmp_copy_paste};
-    #local $SIG{INT} = sub { unlink $file; exit };
+    my $file = $sf->{tmp_copy_paste};
+    local $SIG{INT} = sub { unlink $file; exit };
     if ( ! eval {
         open my $fh_in, '>', $file or die $!;
         # STDIN
@@ -322,7 +328,7 @@ sub from_copy_and_paste {
         die "Error __parse_file!" if ! $ok;
         1 }
     ) {
-        my $ax  = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o} );
+        my $ax  = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
         $ax->print_error_message( $@, join ', ', @$stmt_typeS, 'copy & paste' );
         unlink $file or warn $!;
         return;
@@ -389,8 +395,8 @@ sub from_file {
 
 sub __parse_file {
     my ( $sf, $sql, $stmt_typeS, $file, $fh, $parse_mode ) = @_;
-    #local $SIG{INT} = sub { unlink $sf->{i}{tmp_copy_paste}; exit };
-    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o} );
+    local $SIG{INT} = sub { unlink $sf->{tmp_copy_paste}; exit };
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     $ax->print_sql( $sql, $stmt_typeS );
     print 'Parsing file ...', "\n";
     if ( $parse_mode == 0 ) {
@@ -479,7 +485,7 @@ sub __parse_file {
 
 sub __input_filter {
     my ( $sf, $sql, $stmt_typeS ) = @_;
-    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o} );
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $stmt_h = Term::Choose->new( $sf->{i}{lyt_stmt_h} );
     my $r2c = 0;
     my $backup = [];
