@@ -6,44 +6,53 @@ use strict;
 use 5.008003;
 no warnings 'utf8';
 
-our $VERSION = '2.006';
+our $VERSION = '2.007';
 
 use List::MoreUtils   qw( any );
 
 use Term::Choose           qw( choose );
-use Term::Choose::LineFold qw(print_columns);
+use Term::Choose::LineFold qw( print_columns );
 use Term::Choose::Util     qw( choose_a_file choose_a_subset choose_a_number insert_sep term_width );
 use Term::Form             qw();
 
+use if $^O eq 'MSWin32', 'Win32::Console::ANSI'; #
+
 use App::DBBrowser::Auxil;
 
-use if $^O eq 'MSWin32', 'Win32::Console::ANSI'; #
+
+use App::DBBrowser::Subqueries; ###
 
 
 sub new {
-    my ( $class, $info, $opt ) = @_;
+    my ( $class, $info, $options, $data ) = @_;
     my $sf = {
         i => $info,
-        o => $opt
+        o => $options,
+        d => $data,
+        aggregate    => [ "AVG(X)", "COUNT(X)", "COUNT(*)", "MAX(X)", "MIN(X)", "SUM(X)" ],
+        opr_subquery => [ "IN", "NOT IN", " = ", " != ", " < ", " > ", " >= ", " <= " ],
+        distinct => "DISTINCT",
+        all     => "ALL",
+        asc     => "ASC",
+        desc    => "DESC",
+        and     => "AND",
+        or      => "OR",
     };
-    $sf->{i}{aggregate} = [ "AVG(X)", "COUNT(X)", "COUNT(*)", "MAX(X)", "MIN(X)", "SUM(X)" ]; ###
-    $sf->{i}{opr_subquery} = [ "IN", "NOT IN", " = ", " != ", " < ", " > ", " >= ", " <= " ];
-    # my ( $DISTINCT, $ALL, $ASC, $DESC, $AND, $OR ) = ( "DISTINCT", "ALL", "ASC", "DESC", "AND", "OR" );
     bless $sf, $class;
 }
 
 
 sub __add_aggregate_substmt {
-    my ( $sf, $dbh, $sql, $tmp,  $stmt_type, $cols_type ) = @_;
+    my ( $sf, $sql, $tmp,  $stmt_type, $cols_type ) = @_;
     my $stmt_h = Term::Choose->new( $sf->{i}{lyt_stmt_h} );
-    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o} );
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my @pre = ( undef );
     push @pre, $sf->{i}{ok} if $cols_type eq 'aggr_cols';
     my $i = @{$tmp->{$cols_type}};
     $ax->print_sql( $sql, [ $stmt_type ], $tmp );
     # Choose
     my $aggr = $stmt_h->choose(
-        [ @pre, @{$sf->{i}{aggregate}} ]
+        [ @pre, @{$sf->{aggregate}} ]
     );
     if ( ! defined $aggr ) {
         return;
@@ -58,17 +67,16 @@ sub __add_aggregate_substmt {
         $aggr =~ s/\(\S\)\z//; #
         $tmp->{$cols_type}[$i] = $aggr . "(";
         if ( $aggr eq 'COUNT' ) {
-            my ( $DISTINCT, $ALL ) = ( "DISTINCT", "ALL" ); ##
             $ax->print_sql( $sql, [ $stmt_type ], $tmp );
             # Choose
             my $all_or_distinct = $stmt_h->choose(
-                [ undef, $ALL, $DISTINCT ]
+                [ undef, $sf->{all}, $sf->{distinct} ]
             );
             if ( ! defined $all_or_distinct ) {
                 return;
             }
-            if ( $all_or_distinct eq $DISTINCT ) {
-                $tmp->{$cols_type}[$i] .= $DISTINCT . ' '; #
+            if ( $all_or_distinct eq $sf->{distinct} ) {
+                $tmp->{$cols_type}[$i] .= $sf->{distinct} . ' '; #
             }
         }
         $ax->print_sql( $sql, [ $stmt_type ], $tmp );
@@ -81,9 +89,9 @@ sub __add_aggregate_substmt {
         }
         $tmp->{$cols_type}[$i] .= $f_col . ")";
     }
-    my $alias = $ax->__alias( $dbh, $tmp->{$cols_type}[$i] );
+    my $alias = $ax->alias( $tmp->{$cols_type}[$i] );
     if ( defined $alias && length $alias ) {
-        $tmp->{alias}{$tmp->{$cols_type}[$i]} = $ax->quote_col_qualified( $dbh, [ $alias ] );
+        $tmp->{alias}{$tmp->{$cols_type}[$i]} = $ax->quote_col_qualified( [ $alias ] );
     }
     return 1;
 }
@@ -91,8 +99,8 @@ sub __add_aggregate_substmt {
 
 
 sub columns {
-    my ( $sf, $dbh, $stmt_h, $sql, $stmt_type ) = @_;
-    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o} );
+    my ( $sf, $stmt_h, $sql, $stmt_type ) = @_;
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $tmp = {
         chosen_cols    => [],
         select_type    => 'chosen_cols',
@@ -100,11 +108,11 @@ sub columns {
         alias          => { %{$sql->{alias}} },
     };
     my $aggr_func = '&';
-    my $sub_query = '(Q';
+    my $sq_col = '(Q';
     my $bu = [];
     my @pre = ( undef, $sf->{i}{ok} );
-    push @pre, $aggr_func if $sf->{o}{G}{expert_aggregate};
-    push @pre, $sub_query if $sf->{o}{G}{expert_subqueries};
+    push @pre, $aggr_func if $sf->{o}{G}{aggregate_select};
+    push @pre, $sq_col if $sf->{o}{G}{subqueries};
 
     COLUMNS: while ( 1 ) {
         my $choices = [ @pre, @{$sql->{cols}} ];
@@ -127,28 +135,31 @@ sub columns {
             return $tmp;
         }
         push @$bu, [ [ @{$tmp->{chosen_cols}} ], [ @{$tmp->{select_sq_args}} ], { %{$tmp->{alias}} } ];
-        if ( $col eq $sub_query ) {
-            my $idx = $sf->__choose_hist_idx();
-            if ( ! defined $idx ) {
+        if ( $col eq $sq_col ) {
+            my $sq = App::DBBrowser::Subqueries->new( $sf->{i}, $sf->{o}, $sf->{d} );
+            my $res = $sq->choose_subquery( $sql, $tmp, $stmt_type );
+            if ( ! defined $res ) {
                 ( $tmp->{chosen_cols}, $tmp->{select_sq_args}, $tmp->{alias} ) = @{pop @$bu};
                 next COLUMNS;
             }
-            my $subquery = "(" . $sf->{i}{stmt_history}[$idx][0] . ")"; #
-            my $values   = $sf->{i}{stmt_history}[$idx][1];
-            push @{$tmp->{chosen_cols}}, $subquery ;
+            my ( $subquery, $values ) = @$res;
+            $subquery = "(" . $subquery . ")";
+            push @{$tmp->{chosen_cols}}, $subquery;
             push @{$tmp->{select_sq_args}}, @$values; #
             my $filled = $subquery;
             for my $val ( @$values ) {
                 $filled =~ s/\?/$val/;
             }
-            my $alias = $ax->__alias( $dbh, $filled );
+            my $alias = $ax->alias( $filled );
+            $filled .= '_' . scalar @$values if @$values;
             if ( defined $alias && length $alias ) {
-                $tmp->{alias}{$filled} = $ax->quote_col_qualified( $dbh, [ $alias ] );
+                my $alias_key = $ax->alias_key( $subquery, $values );
+                $tmp->{alias}{$alias_key} = $ax->quote_col_qualified( [ $alias ] );
             }
             next COLUMNS;
         }
         elsif ( $col eq $aggr_func ) {
-            my $ret = $sf->__add_aggregate_substmt( $dbh, $sql, $tmp, $stmt_type, 'chosen_cols' );
+            my $ret = $sf->__add_aggregate_substmt( $sql, $tmp, $stmt_type, 'chosen_cols' );
             if ( ! $ret ) {
                 ( $tmp->{chosen_cols}, $tmp->{select_sq_args}, $tmp->{alias} ) = @{pop @$bu};
                 next COLUMNS;
@@ -160,15 +171,14 @@ sub columns {
     }
 }
 
-sub distinc {
+sub distinct {
     my ( $sf, $stmt_h, $sql, $stmt_type ) = @_;
-    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o} );
-    my ( $DISTINCT, $ALL ) = ( "DISTINCT", "ALL" );
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my @pre = ( undef, $sf->{i}{ok} );
     my $tmp = { distinct_stmt => '' };
 
     DISTINCT: while ( 1 ) {
-        my $choices = [ @pre, $DISTINCT, $ALL ];
+        my $choices = [ @pre, $sf->{distinct}, $sf->{all} ];
         $ax->print_sql( $sql, [ $stmt_type ], $tmp );
         # Choose
         my $select_distinct = $stmt_h->choose(
@@ -189,7 +199,7 @@ sub distinc {
 }
 
 sub aggregate {
-    my ( $sf, $dbh, $stmt_h, $sql, $stmt_type ) = @_;
+    my ( $sf, $stmt_h, $sql, $stmt_type ) = @_;
     my $group_by_cols = $sql->{group_by_cols};
     my $tmp = {
         aggr_cols   => [],
@@ -197,7 +207,7 @@ sub aggregate {
     };
 
     AGGREGATE: while ( 1 ) {
-        my $ret = $sf->__add_aggregate_substmt( $dbh, $sql, $tmp, $stmt_type, 'aggr_cols' );
+        my $ret = $sf->__add_aggregate_substmt( $sql, $tmp, $stmt_type, 'aggr_cols' );
         if ( ! $ret ) {
             if ( @{$tmp->{aggr_cols}} ) {
                 my $aggr = pop @{$tmp->{aggr_cols}};
@@ -218,7 +228,7 @@ sub aggregate {
 
 sub set {
     my ( $sf, $stmt_h, $sql, $stmt_type ) = @_;
-    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o} );
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $trs = Term::Form->new();
     my $col_sep = ' ';
     my $tmp = {
@@ -248,26 +258,19 @@ sub set {
             return $tmp;
         }
         push @$bu, [ [@{$tmp->{set_args}}], $tmp->{set_stmt}, $col_sep ];
-        $tmp->{set_stmt} .= $col_sep . $col . ' =';
-        $ax->print_sql( $sql, [ $stmt_type ], $tmp );
-        # Readline
-        my $value = $trs->readline( $col . ': ' );
-        if ( ! defined $value ) {
-            if ( @$bu ) {
-                ( $tmp->{set_args}, $tmp->{set_stmt}, $col_sep ) = @{pop @$bu};
-                next SET;
-            }
-            return;
+        $tmp->{set_stmt} .= $col_sep . $col;
+        my $ok = $sf->__set_operator_sql( $sql, $tmp, 'set', $col, $stmt_type );
+        if ( ! $ok ) {
+            ( $tmp->{set_args}, $tmp->{set_stmt}, $col_sep ) = @{pop @$bu};
+            next SET;
         }
-        $tmp->{set_stmt} .= ' ' . '?';
-        push @{$tmp->{set_args}}, $value;
         $col_sep = ', ';
     }
 }
 
 sub where {
     my ( $sf, $stmt_h, $sql, $stmt_type ) = @_;
-    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o} );
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my @cols = ( @{$sql->{cols}}, @{$sql->{modified_cols}} );
     my $AND_OR = ' ';
     my $tmp = {
@@ -279,7 +282,7 @@ sub where {
     my $bu = [];
     my $sq_col = '(Q';
     my @pre = ( undef, $sf->{i}{ok} );
-    push @pre, $sq_col if $sf->{o}{G}{expert_subqueries};
+    push @pre, $sq_col if $sf->{o}{G}{subqueries};
 
     WHERE: while ( 1 ) {
         my @choices = ( @cols );
@@ -291,17 +294,6 @@ sub where {
         my $quote_col = $stmt_h->choose(
             [ @pre, @choices ]
         );
-        if ( defined $quote_col && $quote_col eq $sq_col ) {
-            my $idx = $sf->__choose_hist_idx();
-            if ( ! defined $idx ) {
-                $quote_col = undef;
-                next WHERE; ####
-            }
-            else {
-                $quote_col = "(" . $sf->{i}{stmt_history}[$idx][0] . ")"; #
-                push @{$tmp->{where_args}}, @{$sf->{i}{stmt_history}[$idx][1]}; #
-            }
-        }
         if ( ! defined $quote_col ) {
             if ( @$bu ) {
                 ( $tmp->{where_args}, $tmp->{where_stmt}, $AND_OR, $unclosed, $count ) = @{pop @$bu};
@@ -315,6 +307,21 @@ sub where {
             }
             return $tmp;
         }
+        if ( $quote_col eq $sq_col ) {
+            my $sq = App::DBBrowser::Subqueries->new( $sf->{i}, $sf->{o}, $sf->{d} );  # sub
+            my $res = $sq->choose_subquery( $sql, $tmp, $stmt_type );
+            if ( ! defined $res ) {
+                if ( @$bu ) {
+                    ( $tmp->{where_args}, $tmp->{where_stmt}, $AND_OR, $unclosed, $count ) = @{pop @$bu};
+                    next WHERE;
+                }
+                return;
+            }
+            my ( $subquery, $values ) = @$res;
+            #$tmp->{where_stmt} .= " (" . $subquery . ")";
+            $quote_col = "(" . $subquery . ")";
+            push @{$tmp->{where_args}}, @$values; #
+        }
         if ( $quote_col eq ')' ) {
             push @$bu, [ [@{$tmp->{where_args}}], $tmp->{where_stmt}, $AND_OR, $unclosed, $count ];
             $tmp->{where_stmt} .= ")";
@@ -322,11 +329,10 @@ sub where {
             next WHERE;
         }
         if ( $count > 0 && $tmp->{where_stmt} !~ /\(\z/ ) { #
-            my ( $AND, $OR ) = ( "AND", "OR" );
             $ax->print_sql( $sql, [ $stmt_type ], $tmp );
             # Choose
             $AND_OR = $stmt_h->choose(
-                [ undef, $AND, $OR ]
+                [ undef, $sf->{and}, $sf->{or} ]
             );
             if ( ! defined $AND_OR ) {
                 next WHERE;
@@ -344,7 +350,7 @@ sub where {
         $tmp->{where_stmt} .= $AND_OR . $quote_col;
         my $ok = $sf->__set_operator_sql( $sql, $tmp, 'where', $quote_col, $stmt_type );
         if ( ! $ok ) {
-                ( $tmp->{where_args}, $tmp->{where_stmt}, $AND_OR, $unclosed, $count ) = @{pop @$bu};
+            ( $tmp->{where_args}, $tmp->{where_stmt}, $AND_OR, $unclosed, $count ) = @{pop @$bu};
             next WHERE;
         }
         $count++;
@@ -353,7 +359,7 @@ sub where {
 
 sub group_by {
     my ( $sf, $stmt_h, $sql, $stmt_type ) = @_;
-    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o} );
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my @pre = ( undef, $sf->{i}{ok} );
     my $tmp = {
         group_by_stmt => " GROUP BY",
@@ -393,7 +399,7 @@ sub group_by {
 
 sub having {
     my ( $sf, $stmt_h, $sql, $stmt_type ) = @_;
-    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o} );
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $aggr_cols = $sql->{aggr_cols};
     my @pre = ( undef, $sf->{i}{ok} );
     my $AND_OR = ' ';
@@ -406,7 +412,7 @@ sub having {
     my $bu = [];
 
     HAVING: while ( 1 ) {
-        my @choices = ( @{$sf->{i}{aggregate}}, map( '@' . $_, @$aggr_cols ) ); #
+        my @choices = ( @{$sf->{aggregate}}, map( '@' . $_, @$aggr_cols ) ); #
         if ( $sf->{o}{G}{parentheses} == 1 ) {
             unshift @choices, $unclosed ? ')' : '(';
         }
@@ -435,11 +441,10 @@ sub having {
             next HAVING;
         }
         if ( $count > 0 && $tmp->{having_stmt} !~ /\(\z/ ) {
-            my ( $AND, $OR ) = ( "AND", "OR" );
             $ax->print_sql( $sql, [ $stmt_type ], $tmp );
             # Choose
             $AND_OR = $stmt_h->choose(
-                [ undef, $AND, $OR ]
+                [ undef, $sf->{and}, $sf->{or} ]
             );
             if ( ! defined $AND_OR ) {
                 next HAVING;
@@ -492,7 +497,7 @@ sub having {
 
 sub order_by {
     my ( $sf, $stmt_h, $sql, $stmt_type ) = @_;
-    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o} );
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my @pre = ( undef, $sf->{i}{ok} );
     my @cols;
     if ( $sql->{select_type} eq '*' || $sql->{select_type} eq 'chosen_cols' ) {
@@ -535,11 +540,10 @@ sub order_by {
         }
         push @$bu, [ $tmp->{order_by_stmt}, $col_sep ];
         $tmp->{order_by_stmt} .= $col_sep . $col;
-        my ( $ASC, $DESC ) = ( "ASC", "DESC" );
         $ax->print_sql( $sql, [ $stmt_type ], $tmp );
         # Choose
         my $direction = $stmt_h->choose(
-            [ undef, $ASC, $DESC ]
+            [ undef, $sf->{asc}, $sf->{desc} ]
         );
         if ( ! defined $direction ){
             ( $tmp->{order_by_stmt}, $col_sep ) = @{pop @$bu}; #
@@ -552,7 +556,7 @@ sub order_by {
 
 sub limit_offset {
     my ( $sf, $stmt_h, $sql, $stmt_type ) = @_;
-    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o} );
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my @pre = ( undef, $sf->{i}{ok} );
     my $tmp = {
         limit_stmt  => '',
@@ -594,9 +598,9 @@ sub limit_offset {
         }
         if ( $choice eq $offset ) {
             if ( ! $tmp->{limit_stmt} ) {
-                $tmp->{limit_stmt} = " LIMIT " . ( $sf->{o}{G}{max_rows} || '9223372036854775807'  ) if $sf->{i}{driver} eq 'SQLite';   # 2 ** 63 - 1
+                $tmp->{limit_stmt} = " LIMIT " . ( $sf->{o}{G}{max_rows} || '9223372036854775807'  ) if $sf->{d}{driver} eq 'SQLite';   # 2 ** 63 - 1
                 # MySQL 5.7 Reference Manual - SELECT Syntax - Limit clause: SELECT * FROM tbl LIMIT 95,18446744073709551615;
-                $tmp->{limit_stmt} = " LIMIT " . ( $sf->{o}{G}{max_rows} || '18446744073709551615' ) if $sf->{i}{driver} eq 'mysql';    # 2 ** 64 - 1
+                $tmp->{limit_stmt} = " LIMIT " . ( $sf->{o}{G}{max_rows} || '18446744073709551615' ) if $sf->{d}{driver} eq 'mysql';    # 2 ** 64 - 1
             }
             $tmp->{offset_stmt} = " OFFSET";
             $ax->print_sql( $sql, [ $stmt_type ], $tmp );
@@ -614,62 +618,53 @@ sub limit_offset {
 }
 
 
-sub __choose_hist_idx {
-    my ( $sf ) = @_;
-    my @choices;
-    for my $e ( @{$sf->{i}{stmt_history}} ) {
-        my $stmt = $e->[0];
-        for my $val ( @{$e->[1]} ) {
-            $stmt =~ s/\?/$val/;
-        }
-        push @choices, $stmt;
-    }
-    my @pre = ( undef );
-    HIST: while ( 1 ) {
-        my $idx = choose( [ @pre, @choices ], { layout => 3, index => 1, undef => 'BACK' } ); ##
-        if ( ! $idx ) {
-            return;
-        }
-        $idx -= @pre;
-        if ( print_columns( $choices[$idx] ) > term_width() ) {
-            my $ok = choose(
-                [ undef, 'Confirm' ],
-                { prompt => $choices[$idx], undef => '<<' }
-            );
-            if ( ! $ok ) {
-                next HIST;
-            }
-        }
-        return $idx;
-    }
-}
-
-
 sub __set_operator_sql {
     my ( $sf, $sql, $tmp, $clause, $quote_col, $stmt_type ) = @_;
-    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o} );
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $stmt_h = Term::Choose->new( $sf->{i}{lyt_stmt_h} );
     my $trs = Term::Form->new();
     my $stmt = $clause . '_stmt';
     my $args = $clause . '_args';
     my $op_query = '=(Q';
     my @pre = ( undef );
-    push @pre, $op_query if $sf->{o}{G}{expert_subqueries};
+    my @operators;
+    my @opr_subquery;
+    if ( $clause eq 'set' ) {
+        @operators    = ( ' = ' );
+        @opr_subquery = ( ' = ' );
+        unshift @operators, $op_query if $sf->{o}{G}{subqueries};
+    }
+    else {
+        @operators    = @{$sf->{o}{G}{operators}};
+        @opr_subquery = @{$sf->{opr_subquery}};
+        unshift @operators, $op_query if $sf->{o}{G}{subqueries};
+    }
 
     OPERATOR: while( 1 ) {
         my $hist = 0;
-        $ax->print_sql( $sql, [ $stmt_type ], $tmp );
-        # Choose
-        my $operator = $stmt_h->choose( [ @pre, @{$sf->{o}{G}{operators}} ] );
-        if ( ! defined $operator ) {
-            return;
+        my $operator;
+        if ( @operators == 1 ) {
+            $operator = $operators[0];
         }
-        elsif ( $operator eq $op_query ) {
-            $hist = 1;
-            $operator = $stmt_h->choose( [ undef, @{$sf->{i}{opr_subquery}} ] );
+        else {
+            $ax->print_sql( $sql, [ $stmt_type ], $tmp );
+            # Choose
+            $operator = $stmt_h->choose( [ @pre, @operators ] );
             if ( ! defined $operator ) {
                 return;
             }
+            elsif ( $operator eq $op_query ) {
+                $hist = 1;
+                if ( @opr_subquery == 1 ) {
+                    $operator = $opr_subquery[0];
+                }
+                else {
+                    $operator = $stmt_h->choose( [ undef, @opr_subquery ] );
+                    if ( ! defined $operator ) {
+                        return;
+                    }
+                }
+            }
         }
         my $bu_stmt = $tmp->{$stmt};
         if ( $operator =~ /\s%?col%?\z/ ) {
@@ -693,9 +688,9 @@ sub __set_operator_sql {
             }
             else {
                 if ( ! eval {
-                    my $obj_db = App::DBBrowser::DB->new( $sf->{i}, $sf->{o} );
+                    my $plui = App::DBBrowser::DB->new( $sf->{i}, $sf->{o} );
                     my @el = map { "'$_'" } grep { length $_ } $arg =~ /^(%?)(col)(%?)\z/g;
-                    my $qt_arg = $obj_db->concatenate( \@el );
+                    my $qt_arg = $plui->concatenate( \@el );
                     $qt_arg =~ s/'col'/$quote_col/;
                     $tmp->{$stmt} .= ' ' . $qt_arg;
                     1 }
@@ -719,8 +714,8 @@ sub __set_operator_sql {
             my $do_not_match_regexp = $operator =~ /^NOT/       ? 1 : 0;
             my $case_sensitive      = $operator =~ /REGEXP_i\z/ ? 0 : 1;
             if ( ! eval {
-                my $obj_db = App::DBBrowser::DB->new( $sf->{i}, $sf->{o} );
-                $tmp->{$stmt} .= $obj_db->regexp( $quote_col, $do_not_match_regexp, $case_sensitive );
+                my $plui = App::DBBrowser::DB->new( $sf->{i}, $sf->{o} );
+                $tmp->{$stmt} .= $plui->regexp( $quote_col, $do_not_match_regexp, $case_sensitive );
                 push @{$tmp->{$args}}, $value;
                 1 }
             ) {
@@ -735,13 +730,15 @@ sub __set_operator_sql {
         elsif ( $operator =~ /^(?:NOT\s)?IN\z/ ) {
             $tmp->{$stmt} .= ' ' . $operator;
             if ( $hist ) {
-                my $idx = $sf->__choose_hist_idx();
-                if ( ! defined $idx ) {
+                my $sq = App::DBBrowser::Subqueries->new( $sf->{i}, $sf->{o}, $sf->{d} );  # sub
+                my $res = $sq->choose_subquery( $sql, $tmp, $stmt_type );
+                if ( ! defined $res ) {
                     $tmp->{$stmt} = $bu_stmt;
                     next OPERATOR;
                 }
-                $tmp->{$stmt} .= "(" . $sf->{i}{stmt_history}[$idx][0] . ")"; #
-                push @{$tmp->{$args}}, @{$sf->{i}{stmt_history}[$idx][1]};     #
+                my ( $subquery, $values ) = @$res;
+                $tmp->{$stmt} .= " (" . $subquery . ")";
+                push @{$tmp->{$args}}, @$values; #
             }
             else {
                 my $col_sep = '';
@@ -795,13 +792,15 @@ sub __set_operator_sql {
             $tmp->{$stmt} .= ' ' . $operator;
             $ax->print_sql( $sql, [ $stmt_type ], $tmp );
             if ( $hist ) {
-                my $idx = $sf->__choose_hist_idx();
-                if ( ! defined $idx ) {
+                my $sq = App::DBBrowser::Subqueries->new( $sf->{i}, $sf->{o}, $sf->{d} );  # sub
+                my $res = $sq->choose_subquery( $sql, $tmp, $stmt_type );
+                if ( ! defined $res ) {
                     $tmp->{$stmt} = $bu_stmt;
                     next OPERATOR;
                 }
-                $tmp->{$stmt} .= " (" . $sf->{i}{stmt_history}[$idx][0] . ")"; #
-                push @{$tmp->{$args}}, @{$sf->{i}{stmt_history}[$idx][1]};     #
+                my ( $subquery, $values ) = @$res;
+                $tmp->{$stmt} .= " (" . $subquery . ")";
+                push @{$tmp->{$args}}, @$values; #
             }
             else {
                 my $prompt = $operator =~ /^(?:NOT\s)?LIKE\z/ ? 'Pattern: ' : 'Value: '; #
